//cmdman.java Q1 2004
//Features:
//  Handles commands and their args
//  Handles operator commands
//  Handles squad commands

import java.io.*;

public class cmdman {
	static final int version  = 7;

	//Modules
	static loggers    lm;
	static data       pd;
	static billerprot core;
	static tcpserver  net;
	static messages   mesg;

	static long starttime;

	public cmdman(loggers lm, config cfg, data pd, billerprot core,
		tcpserver net)
	{
		this.lm   = lm;
		this.pd   = pd;
		this.core = core;
		this.net  = net;
		
		this.mesg = new messages(lm, cfg, pd, net, new dbio());
		
		starttime = new java.util.Date().getTime() / 1000;
		//lm.Log("D", "<cmdman> Loaded v" + version);
	}

	public messages getMesg()
	{
		return mesg;
	}

	protected boolean dontlog(String cmd)
	{
		if (cmd.equals("chat")) return true;
		if (cmd.equals("password")) return true;
		if (cmd.equals("passwd")) return true;
		if (cmd.equals("squadcreate")) return true;
		if (cmd.equals("squadjoin")) return true;
		if (cmd.equals("squadpassword")) return true;
		if (cmd.equals("login")) return true;
		if (cmd.equals("blogin")) return true;
		if (cmd.equals("bpassword")) return true;
		if (cmd.equals("message")) return true;
		return false;
	}

	public void cmdHandler(String port, String pid, String cmd, String args)
	{
		if (cmd == null || cmd.length() < 1) return;
		cmd = cmd.toLowerCase();

		String str, name = pd.getNameByPid(port, pid);
		
		//Log command and args (if any)
		str = "'" + cmd + "'";
		if (args.length() > 0)
		{
			if (dontlog(cmd))
			{
				str = str + " '...'";
			}
			else
			{
				str = str + " '" + args + "'";
			}
		}
		lm.LogM("I", "cmdman", "{" + pd.getZoneByPort(port) +
			"} [" + name + "] command " + str);
	
		if (opCmdHandler(port, pid, cmd, args))      return;
		if (squadCmdHandler(port, pid, cmd, args))   return;
		if (messageCmdHandler(port, pid, cmd, args)) return;
		if (chatCmdHandler(port, pid, cmd, args))    return;
		
		//All other commands
		if (cmd.equals("buptime"))
		{
			long now = new java.util.Date().getTime() / 1000;
			long uptime = now - starttime;
			net.SendMessage(port, pid, "uptime: " + uptime);
		}
		else if (cmd.equals("bversion"))
		{
			str = "POCoB " + lm.getAppVersion();
			str += " running on " + System.getProperty("os.name");
			str += " " + System.getProperty("os.version");
			str += ", machine: " + System.getProperty("os.arch");
			net.SendMessage(port, pid, str);
		}
		else if (cmd.equals("find"))
		{
			if (args.length() > 0)
			{
				str = pd.getZone( args );
				if (str == null)
				{
					net.SendMessage(port, pid, "Unknown user.");
				}
				else if (str.equals("Not online."))
				{
					net.SendMessage(port, pid, str);
				}
				else
				{
					args = pd.getName( args ); //Correct case
					net.SendMessage(port, pid, args + " is in " +
						str);
				}
			}
		}
		else if (cmd.equals("password"))
		{
			if (args.length() > 0)
			{
				if (args.length() < 8)
				{
					net.SendMessage(port, pid, "WARNING: " +
						"Weak password");
				}
				pd.setPassword(name, args);
				net.SendMessage(port, pid, "Password " +
					"successfully changed.");
			}
		}
		else if (cmd.equals("userid"))
		{
			net.SendMessage(port, pid, "UserId:" +
				pd.getBillerID( pd.getNameByPid(port, pid) )
				);
		}
		else if (cmd.equals("zone"))
		{
			str = pd.getZone( pd.getNameByPid(port, pid) );
			net.SendMessage(port, pid, str);
		}
		else
		{
			//net.SendMessage(port, pid, "POCoB: Command not " +
			//	"recognised. Operators have been notified.");
			net.SendStaffMessage(port, name + " tried ?" + str);
			lm.Log("E", "<cmdman> Unhandled command: " + str);
		}
	}
	
	public boolean opCmdHandler(String port, String pid, String cmd,
		String args)
	{
		String name = pd.getNameByPid(port, pid);
		boolean bBillerOp = pd.authOperator( name );
		
		if (!bBillerOp) return false;

		if (cmd.equals("ban"))
		{
			//debug
			pid = pd.getPidByName(port, args);
			if (pid != null)
				net.sendRaw(port, "PKICK:" + pid + ":Bam!");
		}
		else if (cmd.equals("bscorereset"))
		{
			net.SendAllRaw("SCORERESET");
			lm.Log("I", "<cmdman> [" + name + "] Resetting scores for all zones");
		}
		else if (cmd.equals("bshutdown"))
		{
			net.SendBroadcast("Billing is going down now!");
			//core.shutdown();
			core.stage = core.STAGE_QUIT;
		}
		else if (cmd.equals("bflush"))
		{
			net.SendMessage(port, pid, "Saving biller data");
			lm.Log("I", "<cmdman> [" + name + "] Saving user.dat");
			pd.flush();
			mesg.flush();
		}
		else if (cmd.equals("broadcast"))
		{
			if (args.length() > 0)
				net.SendBroadcast(args + " --" + name );
		}
		else
		{
			return false;
		}
		
		return true;
	}

	public boolean squadCmdHandler(String port, String pid, String cmd,
		String args)
	{
		String name = pd.getNameByPid(port, pid);
		boolean bSquadOwner = pd.authSquadOwner( name );

		if (cmd.equals("squadkick"))
		{
			if (bSquadOwner)
			{
				if (args.length() > 0)
				{
					/* FIX ME */
					net.SendMessage(port, pid,
						"If player was a member of " +
						"your squad, he has been " +
						"kicked.");
				}
			}
			else
			{
				net.SendMessage(port, pid, "Only squad owner " +
					"can kick players from squad.");
			}
			
		}
		else if (cmd.equals("squadowner"))
		{
			if (args.length() > 0)
			{
				/* FIX ME */
				/* Owner is ... */
			}
		}
		else if (cmd.equals("squadlist"))
		{
			if (bSquadOwner)
			{
				/* FIX ME */
				/* squad: name,name */
			}
			else
			{
				net.SendMessage(port, pid, "Only squad owner " +
					"can list members.");
			}
		}
		else
		{
			return false;
		}
		
		return true;
	}

	public boolean messageCmdHandler(String port, String pid, String cmd,
		String args)
	{
		if (cmd.equals("message"))
		{
			if (args.length() > 2 && args.indexOf(":") > 0)
			{
				String[] part = args.split(":", 2);
				mesg.storeMessage(port, pid, part[0], part[1]);
			}
		}
		else if (cmd.equals("messages"))
		{
			mesg.checkMessages(port, pid);
		}
		else
		{
			return false;
		}
		
		return true;
	}

	public boolean chatCmdHandler(String port, String pid, String cmd,
		String args)
	{
		if (cmd.equals("chat") && args.length() > 0)
		{
			/* FIX ME */
			/* Set channels for this player */
			net.SendMessage(port, pid, "chat not " +
				"implemented");
		}
		else if (cmd.equals("chat"))
		{
			net.SendMessage(port, pid, "channel: name,name");
		}
		else if (cmd.equals("notifychat") || cmd.equals("chatnotify"))
		{
			/* FIX ME */
		}
		else
		{
			return false;
		}
		
		return true;
	}
}
